import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.StringReader;

import java.text.DateFormat;
import java.text.SimpleDateFormat;

import java.util.Date;

import java.util.concurrent.LinkedBlockingQueue;

import org.xml.sax.InputSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

class data {
  public String contents;
  public String name;

  public data(String contents, String name) {
    this.contents = contents;
    this.name     = name;
  }
}

class queue {
  public LinkedBlockingQueue<data>   q   = new LinkedBlockingQueue<data>();
  public LinkedBlockingQueue<String> out = new LinkedBlockingQueue<String>();
}

class output implements Runnable {
  private queue q;

  public output(queue q) {
    this.q    = q;
  }

  public void run() {
    while (true) {
      try {

        String line = q.out.take();

        if (line.equals("*** QUIT ***")) {
          return;
        }

        System.out.println(line);

      } catch (Exception ex) {
        ex.printStackTrace();
      }
    }
  }
}

class producer implements Runnable {

  private queue q;
  private File  root;

  public producer(queue q, File root) {
    this.q    = q;
    this.root = root;
  }

  private void process_item(File item) {
    try {
      BufferedReader r    = new BufferedReader(new FileReader(item));
      String         line = r.readLine();
      String         name = item.getName();

      while (line != null) {
        q.q.add(new data(line, name));
        line = r.readLine();
      }
      r.close();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  private void process_dir(File dir) {
    String[] items = dir.list();
    for (String item : items) {
      File f = new File(dir, item);
      if (f.isDirectory()) {
        process_dir(f);
      } else {
        if (item.endsWith(".atom")) {
          process_item(f);
        }
      }
    }
  }

  public void run() {
    process_dir(root);
    for (int i = 0; i < 5; i++) {
      q.q.add(new data(null, "*** QUIT ***"));
    }
  }

}

class consumer implements Runnable {

  private queue q;
  private DocumentBuilder db; 

  public consumer(queue q) {
    this.q    = q;
    try {
      db = DocumentBuilderFactory.newInstance().newDocumentBuilder();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public void run() {
    while (true) {
      String name = "<ignore>";
      try {

        data   d    = q.q.take();
        String line = d.contents;

        name = d.name;

        if (name.equals("*** QUIT ***")) {
          return;
        }

        int count = 0;

        int pos = line.indexOf("</feed>");
        while (pos != -1) {
          String   feed = line.substring(0, pos+7);
          Document doc  = db.parse(new InputSource(new StringReader(feed)));

          NodeList entries = doc.getElementsByTagName("entry");

          count += entries.getLength();

          line = line.substring(pos+7, line.length());
          pos  = line.indexOf("</feed>");
        }
        q.out.add(name + ": " + count + " entries.");
      } catch (Exception ex) {
        ex.printStackTrace();
        q.out.add(name + ": " + "***ERROR***");
      }
    }
  }
}

public class extract_info {

  public static void main(String[] args) {
    try {

      queue q = new queue();

      Thread o = new Thread(new output(q));
      o.start();
    
      String dir  = System.getProperty("user.dir");
      File   root = new File(dir);

      Thread p = new Thread(new producer(q, root));
      p.start();

      Thread[] consumers = new Thread[6];

      for (int i = 0; i < 6; i++) {
        consumers[i] = new Thread(new consumer(q));
        consumers[i].start();
      }

      p.join();

      for (int i = 0; i < 6; i++) {
        consumers[i].join();
      }

      q.out.add("*** QUIT ***");

      o.join();

    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
}
